let removeQuotes str = String.sub str 1 ((String.length str) - 2);;

type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq

type select = Colon | DoubleColon


type expr =
    Literal of int
  | Str of string
  | Float of float
  | Boolean of bool
  | Id of string
  | Select of expr * (select * expr) list * string * string list (* should be a boolean function *)
  | Binop of expr * op * expr
  | Assign of string * expr
  | Call of string * expr list
  | Lis of expr list
  | Hash of (string * expr) list
  | Undef
  (*| Noexpr*)
and restriction =
  Restriction of expr * op * expr

type stmt =
    Block of stmt list
  | Expr of expr
  | Return of expr
  | If of expr * stmt * stmt
  | Foreach of expr * expr * stmt
  | While of expr * stmt

type func_decl = {
    fname : string;
    formals : string list;
    locals : string list; (*for compiler*)
    body : stmt list;
  }

type gstmt = (*general statement*)
    Stmt of stmt
  | Func of func_decl

type program = gstmt list  (*string list * func_decl list*)

let rec string_of_expr = function
    Literal(l) -> string_of_int l
  | Str(s) -> s
  | Float(f) -> string_of_float f
  | Boolean(b) -> string_of_bool b
  | Id(s) -> s
  | Binop(e1, o, e2) ->
      string_of_expr e1 ^ " " ^
      (match o with
	Add -> "+" | Sub -> "-" | Mult -> "*" | Div -> "/"
      | Equal -> "==" | Neq -> "!="
      | Less -> "<" | Leq -> "<=" | Greater -> ">" | Geq -> ">=") ^ " " ^
      string_of_expr e2
  | Assign(v, e) -> v ^ " = " ^ string_of_expr e
  | Call(f, el) ->
      f ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  (*| Noexpr -> ""*)
  | Select(id, chain, postfix, restrictions) ->
      string_of_expr id ^ "[" ^ String.concat "" (List.map (fun (t, id) -> if t=Colon then ":" ^ string_of_expr id else "::" ^ string_of_expr id) chain) ^ postfix ^ ", " ^ String.concat ", " restrictions ^ "]" (* fix me *)
  | Lis(exprs_opt) -> "[" ^ String.concat "," (List.map string_of_expr exprs_opt) ^ "]"
  | Hash(kvpairs_list) -> let string_of_kvpair = fun (k, v) -> k ^ ":" ^ string_of_expr v in "{" ^ String.concat "," (List.map string_of_kvpair kvpairs_list)  ^ "}"

and string_of_restriction = function
  Restriction(lhs,o,rhs) -> string_of_expr lhs ^ " " ^
    (match o with
      Add -> "+" | Sub -> "-" | Mult -> "*" | Div -> "/"
    | Equal -> "==" | Neq -> "!="
    | Less -> "<" | Leq -> "<=" | Greater -> ">" | Geq -> ">=") ^ " " ^
  string_of_expr rhs

let rec string_of_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Expr(expr) -> string_of_expr expr ^ ";\n";
  | Return(expr) -> "return " ^ string_of_expr expr ^ ";\n";
  | If(e, s, Block([])) -> "if (" ^ string_of_expr e ^ ")\n" ^ string_of_stmt s
  | If(e, s1, s2) ->  "if (" ^ string_of_expr e ^ ")\n" ^
      string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
  | Foreach(e1, e2, s) ->
      "foreach " ^ string_of_expr e1  ^ " : " ^ string_of_expr e2 ^ " do\n" ^
         string_of_stmt s
  | While(e, s) -> "while (" ^ string_of_expr e ^ ") " ^ string_of_stmt s

(*let string_of_vdecl id = "int " ^ id ^ ";\n"*) (*no need for declarations, variables are declared when first used*)

let string_of_fdecl fdecl =
  fdecl.fname ^ "(" ^ String.concat ", " fdecl.formals ^ ")\n{\n" ^
  (* String.concat "" (List.map string_of_vdecl fdecl.locals) ^ *)
  String.concat "" (List.map string_of_stmt fdecl.body) ^
  "}\n"

let string_of_gstmt = function
   Stmt(s) -> string_of_stmt s
  | Func(f) -> string_of_fdecl f

let string_of_program gstmt = (*replace string_of_vdecl with gstmt*)
  String.concat "" (List.map string_of_gstmt gstmt)
  (*^ "\n" ^ String.concat "\n" (List.map string_of_fdecl funcs) *)
