%{ open Ast %}

%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA LBRACKET RBRACKET
%token COLON DOUBLECOLON
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token RETURN IF ELSE WHILE FUNCTION FOREACH THEN CONTINUE BREAK DO /* TO AS ELSEIF FOR */
%token TRUE FALSE
%token NULL
/*%token PUTS GETS*/
%token <int> LITERAL
%token <float> FLOAT
%token <bool> BOOLEAN
%token <string> ID
%token <string> STR
%token EOF

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
/*%left EQ NEQ
%left LT GT LEQ GEQ*/
%nonassoc EQ NEQ
%nonassoc LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE

%start program
%type <Ast.program> program

%%

program:
   /* nothing */ { [] }
 | program gstmt { $2 :: $1 }

gstmt:
   stmt  { Stmt($1) }
 | fdecl { Func($1) }

fdecl:
   FUNCTION ID LPAREN formals_opt RPAREN LBRACE stmt_list RBRACE
     { { fname = $2;
	       formals = $4;
	       locals = [];
	       body = List.rev $7 } }

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
    ID                   { [$1] }
  | formal_list COMMA ID { $3 :: $1 }

stmt_list:
    SEMI /* nothing */  { [] }
  | stmt                { [$1] }
  | stmt_list stmt      { $2 :: $1  }
  | stmt_list SEMI      { $1 }

stmt:
    expr SEMI { Expr($1) }
  | RETURN expr SEMI { Return($2) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) }
  | IF expr THEN stmt %prec NOELSE { If($2, $4, Block([])) }
  | IF expr THEN stmt ELSE stmt    { If($2, $4, $6) }
  /*| FOR expr TO expr DO stmt
     { For($2, $4, $6) }*/
  | FOREACH ID COLON expr DO stmt  { Foreach(Id($2), $4, $6) }
  | WHILE expr DO stmt  { While($2, $4) }

selector_chain:
  selector_list { List.rev $1 }

selector_list:
  selector { [$1] }
  | selector_list selector { $2::$1 }

selector:
  selector_type STR { ($1, Str($2)) }
  | selector_type ID { ($1, Id($2)) }

selector_postfix_opt:
    /* nothing */ { "" }
  | GT { ">" }

selector_restriction_opt:
    /* nothing */ { [] }
  | COMMA selector_restriction_list { List.rev $2 }

selector_restriction_list:
    ID { [$1] }
  | selector_restriction_list COMMA ID { $3::$1 }

selector_type:
  COLON { Colon }
  | DOUBLECOLON { DoubleColon }

expr:
    LITERAL          { Literal($1) }
  | ID               { Id($1) }
  | STR              { Str(removeQuotes $1) }
  | FLOAT            { Float($1) }
  | BOOLEAN          { Boolean($1) }
  | expr PLUS   expr { Binop($1, Add,   $3) }
  | expr MINUS  expr { Binop($1, Sub,   $3) }
  | expr TIMES  expr { Binop($1, Mult,  $3) }
  | expr DIVIDE expr { Binop($1, Div,   $3) }
  | expr EQ     expr { Binop($1, Equal, $3) }
  | expr NEQ    expr { Binop($1, Neq,   $3) }
  | expr LT     expr { Binop($1, Less,  $3) }
  | expr LEQ    expr { Binop($1, Leq,   $3) }
  | expr GT     expr { Binop($1, Greater,  $3) }
  | expr GEQ    expr { Binop($1, Geq,   $3) }
  | MINUS expr       { Binop($2, Mult,  Literal(-1)) }
  | NULL             { Undef }
  | ID ASSIGN expr   { Assign($1, $3) }
  | ID LPAREN actuals_opt RPAREN { Call($1, $3) }
  /*| GETS LPAREN actuals_opt RPAREN { Call("gets", $3) }*/
  | LPAREN expr RPAREN { $2 }
  | ID LBRACKET selector_chain selector_postfix_opt selector_restriction_opt RBRACKET { Select(Id($1), $3, $4, $5) }
  | LBRACKET exprs_opt RBRACKET { Lis($2) }
  | LBRACE kvpairs_opt RBRACE { Hash($2)  } 

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }

exprs_opt:
    /* nothing */ { [] }
  | exprs_list { List.rev $1 }

exprs_list:
    expr                    { [$1] }
  | exprs_list COMMA expr   { $3 :: $1 }

kvpairs_opt:
    /* nothing */ { [] }
  | kvpairs_list  { List.rev $1 }

kvpairs_list:
    STR COLON expr                    { [($1, $3)] }
  | kvpairs_list COMMA STR COLON expr { ($3, $5) :: $1 } 
