{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| ['\"']([^'\r' '\n' '\"']|"\\\"")*['\"'] as sc     { STR(sc) }
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| ';'      { SEMI }
| ":"      { COLON }
| "::"     { DOUBLECOLON }
| '['      { LBRACKET }
| ']'      { RBRACKET }
| ','      { COMMA }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "null"   { NULL }
| "if"     { IF }
| "else"   { ELSE }
(*| "for"    { FOR } *)
| "foreach" { FOREACH }
| "while"  { WHILE }
| "return" { RETURN }
| "break"    { BREAK }
| "continue" { CONTINUE }
| "then"     { THEN }
(*| "to"       { TO } 
| "elseif"   { ELSEIF } *)
| "do"       { DO }
(*| "as"       { AS } *)
(*| "puts"     { PUTS }
| "gets"     { GETS }*)
| "fun"      { FUNCTION }
| ['-']?['0'-'9']+['.']['0'-'9']+ as lxm { FLOAT(float_of_string lxm) }
| "true"|"false" as bool { BOOLEAN(bool_of_string bool) }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
